import "moebius92.CCTimer.CCTimerOptionsButton";
import "moebius92.CCTimer.CCTimerOptionsCCEntry";
import "moebius92.CCTimer.CCTimerOptionsCCLabel";
import "moebius92.CCTimer.CCTimerOptionsControl";
import "moebius92.CCTimer.CCTimerOptionsFixedField";
import "moebius92.CCTimer.CCTimerOptionsQuickslot";

local function isnonneg(x)
	x = tonumber(x);
	if x == nil or x < 0 then
		return false;
	end
	return x;
end

local function isnumber(x)
	if tonumber(x) == nil then
		return false;
	end
	return x;
end

local function nilnumber(x) 
	if x == "?" then
		return nil;
	elseif tonumber(x) == nil then
		return false;
	end
	return x;
end

local function tonilstring(x)
	if x == nil then
		return "?"
	end
	return tostring(x);
end

local function tableCopy(t)
	if type(t) ~= "table" then
		return t;
	end

	local r = { };
	for k, v in pairs(t) do
		r[k] = tableCopy(v);
	end
	return r;
end

local function fnMouseClick(self, args, intersection)
	if self.window.ccSkills[self.shortcut].regimes.options:IsVisible() then
		self:SetBrightness(0.25);
		self.window.ccSkills[self.shortcut].regimes.options:SetVisible(false);
	else
		self:SetBrightness(0.50);
		self.window.ccSkills[self.shortcut].regimes.options:SetVisible(true);
	end
end

local function fnMouseEnter(self, args, intersection)
	if intersection ~= nil then
		intersection:SetBrightness(2.0);
		local t, b = intersection:GetBounds();
		for shortcut, regime in pairs(self.options.highlightRegimes) do
			if shortcut ~= self.shortcut then
				regime:SetBounds(t, b);
				regime:SetVisible(true);
			end
		end
	end
end

local function fnMouseLeave(self, args, intersection)
	if intersection ~= nil then
		intersection:SetBrightness(1.0);
		for _, regime in pairs(self.options.highlightRegimes) do
			regime:SetVisible(false);
		end
	end
end

CCTimerOptions = class(Turbine.UI.Control);

function CCTimerOptions:Constructor(window)
	Turbine.UI.Control.Constructor(self);

	self.window = window;
	self.width = 65;
	self.height = 20;

	self:SetBackColor(Turbine.UI.Color(1.0, 0.0, 0.0, 0.3));

	self.top = { };
	table.insert(self.top, CCTimerOptionsFixedField(self, "Act", self.width, 40, self.height, function(x) if x == nil then return ""; end return string.format("%.3f", x); end));
	table.insert(self.top, CCTimerOptionsFixedField(self, "Key", self.width, 40, self.height, function(x) if x == nil then return ""; end return string.format("%.3f", x); end));
	table.insert(self.top, CCTimerOptionsControl(self, "Calibrate", 150, self.height, function() self.window:ToggleCalibrate(); end));
	self.top[3]:SetColor(0.5);

	self.middle = { };

	self.highlightRegimes = { };

	window.regimes.options = CCTimerRegimeContainer(nil);
	window.regimes.options.shortcut = "options";
	window.regimes.options.options = self;
	window.regimes.options.MouseEnter = fnMouseEnter;
	window.regimes.options.MouseLeave = fnMouseLeave;
	window.regimes.options:SetBrightness(0.50);
	window.regimes.options:SetSize(window.size + 4, window:GetHeight());
	for _, skill in pairs(window.ccSkills) do
		skill.regimes.options = window.regimes.options:AddRegime();
	end
	window.regimesRecovery.options = window.regimes.options:AddRegime();
	window.regimesRecovery.options:SetBounds(window.size + window.seconds * 10, window:GetHeight());
	window.regimesRecovery.options:SetVisible(true);
	self.highlightRegimes.options = window.regimes.options:AddRegime();
	self.highlightRegimes.options:SetBrightness(2.0);
	table.insert(self.middle, CCTimerOptionsCCLabel(self, window.regimes.options, self.width, self.height, window.size, 5, { "Duration", "Pre-root", "Act", "Start", "Stop" }));

	local t = { };
	for shortcut in pairs(window.ccSkills) do
		table.insert(t, shortcut);
	end
	table.sort(t);
	for _, shortcut in ipairs(t) do
		window.regimes[shortcut] = CCTimerRegimeContainer(nil);
		window.regimes[shortcut].shortcut = shortcut;
		window.regimes[shortcut].window = window;
		window.regimes[shortcut].options = self;
		window.regimes[shortcut].MouseClick = fnMouseClick;
		window.regimes[shortcut].MouseEnter = fnMouseEnter;
		window.regimes[shortcut].MouseLeave = fnMouseLeave;
		window.regimes[shortcut]:SetBrightness(0.50);
		window.regimes[shortcut]:SetSize(window.size + 4, window:GetHeight());
		window.ccSkills[shortcut].regimes.entry = window.regimes[shortcut]:AddRegime();
		window.ccSkills[shortcut].regimes.entry:SetVisible(true);
		window.regimesRecovery[shortcut] = window.regimes[shortcut]:AddRegime();
		window.regimesRecovery[shortcut]:SetBounds(window.size + window.seconds * 10, window:GetHeight());
		window.regimesRecovery[shortcut]:SetVisible(true);
		self.highlightRegimes[shortcut] = window.regimes[shortcut]:AddRegime();
		self.highlightRegimes[shortcut]:SetBrightness(2.0);
		if window.config[window.curConfig][shortcut].preroot ~= nil then
			table.insert(self.middle, CCTimerOptionsCCEntry(self, window.regimes[shortcut], self.width, self.height, window.size, 5, shortcut,
			                                                { "duration", "preroot", "act", "start", "stop" },
			                                                { isnonneg, isnonneg, isnumber, nilnumber, nilnumber },
			                                                { tostring, tostring, tostring, tonilstring, tonilstring }));
		else
			table.insert(self.middle, CCTimerOptionsCCEntry(self, window.regimes[shortcut], self.width, self.height, window.size, 5, shortcut,
			                                                { "duration", "", "act", "start", "stop" },
			                                                { isnonneg, isnonneg, isnumber, nilnumber, nilnumber },
			                                                { tostring, tostring, tostring, tonilstring, tonilstring }));
		end
	end

	self.bottom1 = { };
	table.insert(self.bottom1, CCTimerOptionsQuickslot(self, window.size));

	self.setupName = Turbine.UI.Lotro.TextBox();
	self.setupName:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.setupName:SetWantsKeyEvents(true);
	self.setupName.FocusLost = function(self, args)
		local config = self:GetText();

		if config ~= self.parent.window.curConfig then
			self.parent.window.config[config] = self.parent.window.config[self.parent.window.curConfig];
			self.parent.window.config[self.parent.window.curConfig] = nil;
			self.parent.window.curConfig = config;
			self.parent.window:SaveSettings();
		end
	end;
	self.setupName.KeyDown = function(self, args)
		if self:HasFocus() and args.Action == 162 then
			self.parent:Focus();
		end
	end;
	self.setupName:SetMultiline(false);
	self.setupName:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.setupName:SetParent(self);
	self.setupName:SetZOrder(0);
	self.setupName.parent = self;
	self.setupMenu = Turbine.UI.ContextMenu();
	self.setupDropDown = Turbine.UI.Button();
	self.setupDropDown:SetBackground("moebius92/CCTimer/res/dropdown.tga");
	self.setupDropDown:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.setupDropDown:SetSize(self.height - 4, self.height);
	self.setupDropDown:SetZOrder(1);
	self.setupDropDown:SetParent(self);
	self.setupDropDown.parent = self;
	self.setupDropDown.MouseClick = function(self, args)
		local items = self.parent.setupMenu:GetItems();
		local f = function(self, args) local config = self:GetText(); if config ~= self.parent.window.curConfig then self.parent.window:LoadConfig(config); end end;
		local t = { };

		items:Clear();
		for label in pairs(self.parent.window.config) do
			table.insert(t, label);
		end
		table.sort(t);
		for _, label in ipairs(t) do
			local item = Turbine.UI.MenuItem(label);
			item.parent = self.parent;
			item.Click = f;
			items:Add(item);
		end
		self.parent.setupMenu:ShowMenuAt(self.parent.setupName:PointToScreen(0, self.parent.setupName:GetHeight()));
	end;

	self.bottom2 = { };
	table.insert(self.bottom2, CCTimerOptionsButton(self, "Copy", function(self, args)
		local curConfig = self.parent.window.curConfig;
		local name = curConfig .. " (copy)";

		if self.parent.window.config[name] ~= nil then
			local i = 1;
			while true do
				name = curConfig .. " (copy " .. tostring(i) .. ")";
				if self.parent.window.config[name] == nil then
					break;
				end
				i = i + 1;
			end
		end

		self.parent.window.config[name] = tableCopy(self.parent.window.config[curConfig]);
		self.parent.window:SaveSettings();
		self.parent.window:LoadConfig(name);
	end));
	table.insert(self.bottom2, CCTimerOptionsButton(self, "Delete", function(self, args)
		local n = 0;

		for k in pairs(self.parent.window.config) do
			n = n + 1;
		end

		if n > 1 then
			self.parent.window.config[self.parent.window.curConfig] = nil;
			for k in pairs(self.parent.window.config) do
				self.parent.window:LoadConfig(k);
				break;
			end
			self.parent.window:SaveSettings();
		end
	end));

	self:UpdateEntries();
end

function CCTimerOptions:UpdateEntries()
	local config = self.window.config[self.window.curConfig];

	self.setupName:SetText(self.window.curConfig);

	for i, entry in ipairs(self.middle) do
		if i == 1 then
			for field, color in pairs(entry.colors) do
				color:SetBackColor(Turbine.UI.Color(1.0, config[field].r, config[field].g, config[field].b));
			end
		else
			for _, field in pairs(entry.fields) do
				entry:SetValue(field.entry, self.window.config[self.window.curConfig][entry.shortcut][field.entry]);
			end
			entry.color:SetBackColor(Turbine.UI.Color(1.0, config[entry.shortcut].r, config[entry.shortcut].g, config[entry.shortcut].b));

			if self.window.ccSkills[entry.shortcut].regimes.options:IsVisible() then
				self.window.regimes[entry.shortcut]:SetBrightness(0.50);
			else
				self.window.regimes[entry.shortcut]:SetBrightness(0.25);
			end
		end
	end

	if config.skill == nil then
		self.bottom1[1]:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Undefined, ""));
	else
		self.bottom1[1]:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, config.skill));
	end
end

function CCTimerOptions:SetShortcut(value)
	self.window.config[self.window.curConfig].skill = value;
	self.window:SaveSettings();
end

function CCTimerOptions:SetSetting(shortcut, field, value)
	self.window.config[self.window.curConfig][shortcut][field] = value;
	self.window:SaveSettings();
	self.window:UpdateRegimes();
end

function CCTimerOptions:SetColorSetting(shortcut, r, g, b, save)
	self.window.config[self.window.curConfig][shortcut].r = r;
	self.window.config[self.window.curConfig][shortcut].g = g;
	self.window.config[self.window.curConfig][shortcut].b = b;
	if save then
		self.window:SaveSettings();
	end
	self.window:UpdateRegimes();
end

function CCTimerOptions:GetColorSetting(shortcut)
	return self.window.config[self.window.curConfig][shortcut].r, self.window.config[self.window.curConfig][shortcut].g, self.window.config[self.window.curConfig][shortcut].b;
end

function CCTimerOptions:Calibrate(act, key)
	self.top[1]:SetValue(act);
	self.top[2]:SetValue(key);
end

function CCTimerOptions:StartCalibrate()
	self.top[3]:SetColor(1.0);
end

function CCTimerOptions:StopCalibrate(act, key)
	self.top[3]:SetColor(0.5);
end

function CCTimerOptions:Layout(controls, y, h, m)
	local w = self:GetWidth();
	local t = { };
	local q = { };
	local c, i, l, p, x;

	for k, v in ipairs(controls) do
		t[k] = v:GetWidth();
	end

	i = 1;
	c = 0;
	p = { };
	while t[i] ~= nil do
		l = w - 4 * m;
		l = l - t[i];
		repeat
			table.insert(q, { i = i + 1, c = c + (l * l), p = { e = i, p = p } });
			i = i + 1;
			if t[i] ~= nil then
				l = l - (m + t[i]);
			else
				break;
			end
		until l <= 0;
		table.sort(q, function(a, b) return a.c < b.c; end);
		i = table.remove(q, 1);
		c = i.c;
		p = i.p;
		i = i.i;
	end

	q = { };
	while p.p ~= nil do
		table.insert(q, 1, p.e);
		p = p.p;
	end

	c = 1;
	for _, v in ipairs(q) do
		x = w;
		for i = c, v, 1 do
			x = x - t[i];
		end
		x = (x - (v - c) * m) / 2;
		for i = c, v, 1 do
			controls[i]:SetPosition(x, y);
			x = x + t[i] + m;
		end

		y = y + h + m;
		c = v + 1;
	end
	return y;
end

function CCTimerOptions:SizeChanged()
	local w, h = self:GetSize();

	if self.w ~= w then
		local y = 10;

		y = self:Layout(self.top, y, self.height, 10) + 10;

		y = self:Layout(self.middle, y, self.middle[1]:GetHeight(), 15) + 10;

		y = self:Layout(self.bottom1, y, self.window.size, 10) + 10;

		local l = math.min(w - 4, self.width * 3.5);
		x = (w - l) / 2;
		self.setupName:SetSize(l, self.height);
		self.setupName:SetPosition(x, y);
		x = x + (l - (self.height - 1));
		self.setupDropDown:SetPosition(x, y);
		y = y + self.height + 5;

		y = self:Layout(self.bottom2, y, self.window.size, 5) + 5;

		self:SetHeight(y);
		self.w = w;
	end
end