CCTimerOptionsColor = class(Turbine.UI.Window);

function CCTimerOptionsColor:Constructor(parent)
	Turbine.UI.Window.Constructor(self);

	self.parent = parent;
	self:SetSize(95, 95);
	self:SetMouseVisible(true);
	self:SetBackground("moebius92/CCTimer/res/color.tga");
end

function CCTimerOptionsColor:CalculateColor(x, y)
	local dx = x - 47;
	local dy = y - 47;
	local s = math.sqrt(dx * dx + dy * dy);

	if s <= 39 then
		s = math.min(1.0, s / 37);
		if s == 0 then
			return 0.0, 0.0, 0.0;
		end

		local rgb = { };
		rgb[0] = 1.0 - s;
		rgb[1] = 1.0 - s;
		rgb[2] = 1.0 - s;

		h = ((math.atan2(dx / s, dy / s) * 180 / math.pi + 90) % 360) / 60;
		h = (h + 1) % 6;
		local x = math.floor(h / 2);
		rgb[x] = rgb[x] + s;
		h = h % 2;
		if h < 1 then
			x = (x - 1) % 3;
			rgb[x] = rgb[x] + s * (1.0 - h);
		else
			x = (x + 1) % 3;
			rgb[x] = rgb[x] + s * (h - 1.0);
		end
		return rgb[0], rgb[1], rgb[2];
	end
	return nil, nil, nil;
end

function CCTimerOptionsColor:MouseMove(args)
	local r, g, b = self:CalculateColor(args.X, args.Y);
	if r ~= nil then
		self.parent.target:SetBackColor(Turbine.UI.Color(1.0, r, g, b));
		self.parent.target.options:SetColorSetting(self.parent.target.shortcut, r, g, b, false);
	else
		self.parent.target:SetBackColor(Turbine.UI.Color(1.0, self.r, self.g, self.b));
		self.parent.target.options:SetColorSetting(self.parent.target.shortcut, self.r, self.g, self.b, false);
	end
end

function CCTimerOptionsColor:MouseLeave(args)
	self.parent.target:SetBackColor(Turbine.UI.Color(1.0, self.r, self.g, self.b));
	self.parent.target.options:SetColorSetting(self.parent.target.shortcut, self.r, self.g, self.b, false);
end

function CCTimerOptionsColor:FocusGained(args)
	local r, g, b = self.parent.target.options:GetColorSetting(self.parent.target.shortcut);
	self.r = r;
	self.g = g;
	self.b = b;
end

function CCTimerOptionsColor:FocusLost(args)
	self.parent.target:SetBackColor(Turbine.UI.Color(1.0, self.r, self.g, self.b));
	self:SetVisible(false);
end

function CCTimerOptionsColor:MouseClick(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		local r, g, b = self:CalculateColor(args.X, args.Y);
		if r ~= nil then
			self.r = r;
			self.g = g;
			self.b = b;
			self.parent.target:SetBackColor(Turbine.UI.Color(1.0, r, g, b));
			self.parent.target.options:SetColorSetting(self.parent.target.shortcut, r, g, b, true);
		else
			self.parent.target:SetBackColor(Turbine.UI.Color(1.0, self.r, self.g, self.b));
		end
		self:SetVisible(false);
	end
end

CCTimerOptionsColorSelect = class(Turbine.UI.Control);

function CCTimerOptionsColorSelect:Constructor(parent, options, shortcut, s)
	Turbine.UI.Control.Constructor(self);

	self.options = options;
	self.shortcut = shortcut;

	self:SetParent(parent);
	self:SetSize(s, s);
end

function CCTimerOptionsColorSelect:MouseClick(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		CCTimerOptionsColorSelect.target = self;
		CCTimerOptionsColorSelect.color:SetPosition(Turbine.UI.Display.GetMousePosition());
		CCTimerOptionsColorSelect.color:SetVisible(true);
		CCTimerOptionsColorSelect.color:Activate();
		CCTimerOptionsColorSelect.color:Focus();
	end
end

CCTimerOptionsColorSelect.color = CCTimerOptionsColor(CCTimerOptionsColorSelect);