CCTimerRegime = class();

function CCTimerRegime:Constructor(parent)
	self.parent = parent;

	self.brightness = 1.0;

	self.top = nil;
	self.bottom = nil;

	self.visible = false;

	self.shown = false;
end

function CCTimerRegime:SetColor(r, g, b)
	self.r = r;
	self.g = g;
	self.b = b;

	self.parent:ColorUpdate();
end

function CCTimerRegime:SetBrightness(brightness)
	self.brightness = brightness;

	self.parent:ColorUpdate();
end

function CCTimerRegime:SetBounds(top, bottom)
	if self.top ~= top or self.bottom ~= bottom then
		self.top = top;
		self.bottom = bottom;

		self.parent:IntersectionUpdate();
	end
end

function CCTimerRegime:GetBounds()
	return self.top, self.bottom;
end

function CCTimerRegime:SetVisible(visible)
	if self.visible ~= visible then
		self.visible = visible;

		self.parent:IntersectionUpdate();
	end
end

function CCTimerRegime:IsVisible()
	return self.visible;
end

CCTimerRegimeIntersection = class(Turbine.UI.Control);

function CCTimerRegimeIntersection:Constructor(parent, regimes)
	Turbine.UI.Control.Constructor(self);

	self.parent = parent;
	self.regimes = regimes;
	self.brightness = 1.0;

	self:SetParent(parent);
	self:SetWidth(parent:GetWidth());
	self:SetLeft(0);
	self:ColorUpdate();
	self:SetMouseVisible(parent:IsMouseVisible());
end

function CCTimerRegimeIntersection:Destructor()
	self:SetParent(nil);
end

function CCTimerRegimeIntersection:SetBrightness(brightness)
	self.brightness = brightness;
	self:ColorUpdate();
end

function CCTimerRegimeIntersection:GetBounds()
	if #self.regimes == 1 then
		local t, b = self.regimes[1]:GetBounds();

		if t == nil or b == nil then
			return t, b;
		end
	end

	local t = self:GetTop();
	local h = self:GetHeight();
	return t, (t + h - 1);
end

function CCTimerRegimeIntersection:ColorUpdate()
	local r = 0;
	local g = 0;
	local b = 0;
	local n = 0;
	local brightness = self.brightness * self.parent.brightness;

	for _, regime in ipairs(self.regimes) do
		if regime.r ~= nil then
			r = r + regime.r;
			g = g + regime.g;
			b = b + regime.b;
			n = n + 1;
		end
		brightness = brightness * regime.brightness;
	end

	if n == 1 then
		self:SetBackColor(Turbine.UI.Color(self.parent.alpha, r * brightness, g * brightness, b * brightness));
		self.color = Turbine.UI.Color(1.00, r, g, b);
		self:SetVisible(true);
	elseif n == 0 then
		self:SetVisible(false);
	else
		r = r / n;
		g = g / n;
		b = b / n;
		local m = math.max(r, g, b);

		r = 1.0 - r / m;
		g = 1.0 - g / m;
		b = 1.0 - b / m;
		m = math.max(r, g, b);

		if m > 0 then
			r = r / m;
			g = g / m;
			b = b / m;
		end

		self:SetBackColor(Turbine.UI.Color(self.parent.alpha, r * brightness, g * brightness, b * brightness));
		self.color = Turbine.UI.Color(1.00, r, g, b);
		self:SetVisible(true);
	end
end

function CCTimerRegimeIntersection:MouseClick(args)
	if self.parent.MouseClick ~= nil then
		self.parent:MouseClick(args, self);
	end
end

function CCTimerRegimeIntersection:MouseEnter(args)
	if self.parent.MouseEnter ~= nil then
		self.parent:MouseEnter(args, self);
	end
end

function CCTimerRegimeIntersection:MouseLeave(args)
	if self.parent.MouseLeave ~= nil then
		self.parent:MouseLeave(args, self);
	end
end

CCTimerRegimeContainer = class(Turbine.UI.Control);

function CCTimerRegimeContainer:Constructor(parent)
	Turbine.UI.Control.Constructor(self);

	self.regimes = { };
	self.intersections = { };

	self.alpha = 1.0;
	self.brightness = 1.0;

	self:SetParent(parent);
end

function CCTimerRegimeContainer:SetWidth(w)
	Turbine.UI.Control.SetWidth(self, w);

	for _, intersection in ipairs(self.intersections) do
		intersection:SetWidth(w);
	end
end

function CCTimerRegimeContainer:SetHeight(h)
	Turbine.UI.Control.SetHeight(self, h);

	self:IntersectionUpdate();
end

function CCTimerRegimeContainer:SetSize(w, h)
	self:SetWidth(w);
	self:SetHeight(h);
end

function CCTimerRegimeContainer:SetMouseVisible(v)
	Turbine.UI.Control.SetMouseVisible(self, v);
	for _, intersection in ipairs(self.intersections) do
		intersection:SetMouseVisible(v);
	end
end

function CCTimerRegimeContainer:SetColor(r, g, b)
	self.r = r;
	self.g = g;
	self.b = b;
	self:BackColorUpdate();
end

function CCTimerRegimeContainer:SetAlpha(alpha)
	self.alpha = alpha;
	self:BackColorUpdate();
	self:ColorUpdate();
end

function CCTimerRegimeContainer:SetBrightness(brightness)
	self.brightness = brightness;
	self:BackColorUpdate();
	self:ColorUpdate();
end

function CCTimerRegimeContainer:BackColorUpdate()
	if self.r ~= nil then
		self:SetBackColor(Turbine.UI.Color(self.alpha, self.r * self.brightness, self.g * self.brightness, self.b * self.brightness));
		self.color = Turbine.UI.Color(1.0, self.r * self.brightness, self.g * self.brightness, self.b * self.brightness);
	end
end

function CCTimerRegimeContainer:ColorUpdate()
	for _, intersection in ipairs(self.intersections) do
		intersection:ColorUpdate();
	end
end

function CCTimerRegimeContainer:GetColor(x)
	for _, intersection in ipairs(self.intersections) do
		if intersection.top <= x and (intersection.bottom + 1) > x then
			return intersection.color;
		end
	end
	return self.color;
end

function CCTimerRegimeContainer:AddRegime()
	local r = CCTimerRegime(self);

	table.insert(self.regimes, r);
	return r;
end

function CCTimerRegimeContainer:DeleteRegime(r)
	for i, regime in ipairs(self.regimes) do
		if regime == r then
			table.remove(self.regimes, i);
			if regime.visible and (regime.top ~= nil or regime.bottom ~= nil) then
				self:IntersectionUpdate();
			end
			return;
		end
	end
end

function CCTimerRegimeContainer:IntersectionUpdate()
	for _, intersection in ipairs(self.intersections) do
		intersection:Destructor();
	end
	self.intersections = { };

	local t = { };
	for i, regime in ipairs(self.regimes) do
		if regime.visible and (regime.top ~= nil or regime.bottom ~= nil) then
			if regime.top == nil then
				local b = 0;
				for _, r in ipairs(self.regimes) do
					if r.visible and r.bottom ~= nil and r.bottom < regime.bottom and r.bottom >= b then
						b = r.bottom + 1;
					end
				end
				table.insert(t, { x = b, add = true, regime = i });
			else
				table.insert(t, { x = regime.top, add = true, regime = i });
			end

			if regime.bottom == nil then
				local b = self:GetHeight() - 1;
				for _, r in ipairs(self.regimes) do
					if r.visible and r.top ~= nil and r.top > regime.top and r.top < b then
						b = r.top;
					end
				end
				table.insert(t, { x = b, add = false, regime = i });
			else
				table.insert(t, { x = regime.bottom + 1, add = false, regime = i });
			end
		end
	end
	table.sort(t, function(a, b) return a.x > b.x; end);

	local regimes = { };
	while #t > 0 do
		local x = t[#t].x;
		while #t > 0 and t[#t].x == x do
			local i = table.remove(t);
			if i.add then
				regimes[i.regime] = true;
			else
				regimes[i.regime] = nil;
			end
		end

		if #t > 0 then
			local r = { };
			for i in pairs(regimes) do
				table.insert(r, self.regimes[i]);
			end
			if #r > 0 then
				local i = CCTimerRegimeIntersection(self, r);
				i.top = x;
				i.bottom = t[#t].x - 1;
				i:SetTop(i.top);
				i:SetHeight((i.bottom - i.top) + 1);
				table.insert(self.intersections, i);
			end
		end
	end
end