import "moebius92.SkillData";
import "moebius92.CCTimer.CCTimerRegime";
import "moebius92.CCTimer.CCTimerOptions";

local quickKeys = { 0x1000002A, 0x10000032, 0x1000003E, 0x10000047, 0x10000050, 0x10000057,
                    0x1000005F, 0x10000067, 0x10000071, 0x1000004F, 0x10000056, 0x1000005E,

                    0x10000066, 0x10000070, 0x10000078, 0x10000082, 0x1000008E, 0x10000096,
                    0x1000009E, 0x10000011, 0x10000019, 0x10000022, 0x10000029, 0x10000031,

                    0x1000003D, 0x10000046, 0x1000004E, 0x10000055, 0x1000005D, 0x10000005,
                    0x1000000B, 0x10000010, 0x10000018, 0x10000021, 0x10000028, 0x10000030,

                    0x1000003C, 0x10000045, 0x1000004D, 0x1000008D, 0x10000095, 0x1000009D,
                    0x100000A3, 0x100000AD, 0x100000B0, 0x100000B9, 0x10000004, 0x1000000A,

                    0x1000000F, 0x10000020, 0x10000027, 0x1000002F, 0x1000003B, 0x10000044,
                    0x1000004C, 0x10000054, 0x1000005C, 0x10000065, 0x1000006F, 0x100000AC,

                    0x100000AF, 0x100000B8, 0x10000003, 0x10000009, 0x1000000E, 0x10000017,
                    0x1000001F, 0x10000026, 0x1000002E, 0x1000009C, 0x100000A2, 0x100000AB };

local keymap = { };
for quickslot, key in pairs(quickKeys) do
	keymap[key] = quickslot;
end

local ccEffects = {
	["Dazed"]						= "0x70003152",
	["Feared"]						= "0x70003E82",
	["Recovering"]					= 0,
	["Rooted"]						= "0x70003EC0",
	["Stunned"]						= "0x70003EC2",
	["Vulnerable"]					= "0x70003F0D"
};

local otherEffects = {
	["The Shaking of Arda"]			= "0x70003143",
	["Temporary State Immunity"]	= "0x70003EBC"
};

CCTimerWindow = class(Turbine.UI.Window);

function CCTimerWindow:Constructor(version)
	Turbine.UI.Window.Constructor(self);

	self.size = 36;
	self.seconds = 30;
	self:SetHeight(self.size + (self.seconds + 1) * 10);

	self.x = 0;
	self.y = 0;
	self.dragging = false;
	self.version = version;
	self.config = { };
	self.curConfig = "";

	self.hide = false;
	self.dragging = false;
	self.move = false;

	self.ccSkills = {
		["0x7000B60D"] = { recovery =  true, duration = 15, act = -0.172, start = 2.4, stop = 1.3, r = 1, g = 1, b = 0, effect = "Dazed" },
		["0x70003152"] = { recovery =  true, duration = 30, act =  0.000, start = 1.4, stop = 0.3, r = 1, g = 1, b = 0, effect = "Dazed" },
		["0x70003143"] = { recovery = false, duration = 30, act = -1.250, start = 4.7, stop = nil, r = 0, g = 0, b = 1, effect = "The Shaking of Arda" },
		["0x70003EC0"] = { recovery = false, duration = 30, act = -1.250, start = nil, stop = nil, r = 0, g = 0, b = 1, effect = "Rooted" },
		["0x70000FA7"] = { recovery =  true, duration =  6, act = -1.547, start = 0.0, stop = 0.0, r = 1, g = 0, b = 0, effect = "Stunned" },
		["0x70003EC2"] = { recovery =  true, duration =  3, act = -1.188, start = 1.2, stop = 0.1, r = 1, g = 0, b = 0, effect = "Stunned" },
		["0x7000B63C"] = { recovery =  true, duration =  3, act = -1.234, start = nil, stop = nil, r = 1, g = 0, b = 0, effect = "Stunned" },
		["0x70003EB9"] = { recovery =  true, duration =  5, act = -0.641, start = nil, stop = nil, r = 1, g = 0, b = 0, effect = "Stunned" }	
	};

	self:LoadSettings();

	self.regimes = { };
	for _, skill in pairs(self.ccSkills) do
		skill.regimes = { };
	end
	self.regimesRecovery = { };

	self.options = CCTimerOptions(self);

	self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.name = self.player:GetName();
	self.tskills = self.player:GetTrainedSkills();

	self.ccTrack = { };
	self.regimes.cc = CCTimerRegimeContainer(self);
	self.regimes.cc:SetAlpha(0.25);
	self.regimes.cc:SetHeight(self:GetHeight());
	self.regimes.cc:SetPosition(self.size + 4 + 2, 0);
	self.regimes.cc:SetMouseVisible(false);
	for _, skill in pairs(self.ccSkills) do
		skill.regimes.cc = self.regimes.cc:AddRegime();
	end
	self.regimesRecovery.cc = self.regimes.cc:AddRegime();
	self.regimesRecovery.cc:SetBounds(self.size + self.seconds * 10, self:GetHeight());
	self.regimesRecovery.cc:SetVisible(true);

	self.regimes.target = CCTimerRegimeContainer(self);
	self.regimes.target:SetAlpha(0.25);
	self.regimes.target:SetSize(self.size + 4, self:GetHeight());
	self.regimes.target:SetPosition(0, 0);
	self.regimes.target:SetMouseVisible(false);
	for _, skill in pairs(self.ccSkills) do
		skill.regimes.target = self.regimes.target:AddRegime();
	end
	self.regimesRecovery.target = self.regimes.target:AddRegime();
	self.regimesRecovery.target:SetBounds(self.size + self.seconds * 10, self:GetHeight());
	self.regimesRecovery.target:SetVisible(true);
	self.targetTrack = { };
	self.targetCC = self:NewCCElement(self.regimes.target);

	Turbine.PluginManager.LoadPlugin("VitalsRacer");
	if _G.vitalsRacerWindow ~= nil then
		self.x = Turbine.UI.Display.GetWidth() * 2 / 3;
		self.y = _G.vitalsRacerWindow:GetTop() - self:GetHeight();
		for effect in pairs(ccEffects) do
			_G.vitalsRacerWindow.targetFrame:IgnoreEffect(effect);
		end
		for effect in pairs(otherEffects) do
			_G.vitalsRacerWindow.targetFrame:IgnoreEffect(effect);
		end
	end

	self.fnEffectAdded = function(sender, args)
		local e = self.targetEffects:Get(args.Index);
		local name = e:GetName();

		if self.time ~= nil then
			for _, skill in pairs(self.ccSkills) do
				if skill.effect == name then
					self.time.cc = e:GetStartTime();
					self:Calibrate();
					break;
				end
			end
		end

		if ccEffects[name] ~= nil then
			self.targetCCName = name;
			self.targetCCID = e:GetID();
			if ccEffects[name] ~= 0 then
				self.targetCC.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, ccEffects[name]));
			end
			self.targetCC.startTime = e:GetStartTime();
			self.targetCC.endTime = self.targetCC.startTime + e:GetDuration();
			self.targetCC.control:SetVisible(true);
			self:UpdateLayout();
		elseif otherEffects[name] ~= nil then
			local item = self:NewCCElement(self.regimes.target);
			item.type = otherEffects[name];
			item.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, item.type));
			item.startTime = e:GetStartTime();
			item.endTime = item.startTime + e:GetDuration();
			item.ID = e:GetID();
			item.control:SetVisible(true);
			table.insert(self.targetTrack, item);
			self:UpdateLayout();
		end
	end;
	self.fnEffectRemoved = function(sender, args)
		local id = args.Effect:GetID();

		if id == self.targetCCID then
			self.targetCCName = nil;
			self.targetCCID = nil;
			self.targetCC.control:SetVisible(false);
			self:UpdateLayout();
		else
			for i, effect in ipairs(self.targetTrack) do
				if effect.ID == id then
					effect.control:SetVisible(false);
					effect.control:SetParent(nil);
					table.remove(self.targetTrack, i);
					self:UpdateLayout();
					break;
				end
			end
		end
	end;
	self.fnEffectsCleared = function(sender, args)
		if self.targetCCName ~= nil or #self.targetTrack > 0 then
			self.targetCCName = nil;
			self.targetCCID = nil;
			self.targetCC.control:SetVisible(false);

			for _, effect in ipairs(self.targetTrack) do
				effect.control:SetVisible(false);
				effect.control:SetParent(nil);
			end
			self.targetTrack = { };

			self:UpdateLayout();
		end
	end;

	self:HookSkills();
	moebius92.Handlers.AddChatHandler(function(m) self:ChatMessage(m) end, Turbine.ChatType.PlayerCombat, self.name);
	moebius92.Handlers.AddEventHandler(self.tskills, "SkillAdded", function() self:HookSkills(); end);
	moebius92.Handlers.AddEventHandler(self.tskills, "SkillRemoved", function() self:HookSkills(); end);
	moebius92.Handlers.AddEventHandler(self.player, "TargetChanged", function() self:StopCalibrate(); self:UpdateTarget(); end);

	self:SetPosition(self.x, self.y);
	self:SetWantsKeyEvents(true);
	self:SetMouseVisible(false);
	self:SetVisible(true);
	self:UpdateRegimes();
	self:UpdateLayout();
	self:UpdateTarget();
end

function CCTimerWindow:NewCCElement(parent)
	local element = { };

	element.control = Turbine.UI.Control();
	element.control:SetSize(self.size, self.size);
	element.control:SetZOrder(1);
	element.control:SetVisible(false);
	element.control:SetMouseVisible(false);
	element.control:SetParent(parent);
	element.quickslot = Turbine.UI.Lotro.Quickslot();
	element.quickslot:SetParent(element.control);
	element.quickslot:SetPosition(-1, -1);
	element.quickslot:SetSize(self.size, self.size);
	element.quickslot:SetMouseVisible(false);
	return element;
end

function CCTimerWindow:UpdateTarget()
	if self.target ~= nil then
		for event, handler in pairs(self.targetEffectsHandlers) do
			moebius92.Handlers.RemoveEventHandler(self.targetEffects, event, handler);
		end
	end

	self.target = nil;
	self.targetEffects = nil;
	self.targetEffectsHandlers = nil;
	self.targetCCName = nil;
	self.targetCCID = nil;
	self.targetCC.control:SetVisible(false);

	for _, effect in ipairs(self.targetTrack) do
		effect.control:SetVisible(false);
		effect.control:SetParent(nil);
	end
	self.targetTrack = { };

	self.target = self.player:GetTarget();
	if self.target ~= nil and self.target:IsA(Turbine.Gameplay.Actor) then
		self.targetEffects = self.target:GetEffects();
		self.targetEffectsHandlers = { };
		self.targetEffectsHandlers.EffectAdded = moebius92.Handlers.AddEventHandler(self.targetEffects, "EffectAdded", self.fnEffectAdded);
		self.targetEffectsHandlers.EffectRemoved = moebius92.Handlers.AddEventHandler(self.targetEffects, "EffectRemoved", self.fnEffectRemoved);
		self.targetEffectsHandlers.EffectsCleared = moebius92.Handlers.AddEventHandler(self.targetEffects, "EffectsCleared", self.fnEffectsCleared);

		for i = 1, self.targetEffects:GetCount(), 1 do
			local e = self.targetEffects:Get(i);
			local name = e:GetName();

			if ccEffects[name] ~= nil then
				self.targetCCName = name;
				self.targetCCID = e:GetID();
				if ccEffects[name] ~= 0 then
					self.targetCC.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, ccEffects[name]));
				else
					self.targetCC.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, "0x70003F0D"));
				end
				self.targetCC.startTime = e:GetStartTime();
				self.targetCC.endTime = self.targetCC.startTime + e:GetDuration();
				self.targetCC.control:SetVisible(true);
			elseif otherEffects[name] ~= nil then
				local item = self:NewCCElement(self.regimes.target);
				item.type = otherEffects[name];
				item.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, item.type));
				item.startTime = e:GetStartTime();
				item.endTime = item.startTime + e:GetDuration();
				item.ID = e:GetID();
				item.control:SetVisible(true);
				table.insert(self.targetTrack, item);
			end
		end
	else
		self.target = nil;
	end

	self:UpdateLayout();
end

function CCTimerWindow:HookSkills()
	for _, skill in pairs(self.ccSkills) do
		if skill.activeSkill ~= nil then
			for event, handler in pairs(skill.handlers) do
				moebius92.Handlers.RemoveEventHandler(skill.activeSkill, event, handler);
			end
		end

		skill.handlers = nil;
		skill.activeSkill = nil;
	end

	for _, skill in pairs(self.ccSkills) do
		skill.regimes.cc:SetVisible(false);
		skill.regimes.target:SetVisible(false);
		skill.regimes.options:SetVisible(false);
	end

	local possConfigs = { };
	for i = 1, self.tskills:GetCount(), 1 do
		local askill = self.tskills:GetItem(i);
		local name = askill:GetSkillInfo():GetName();

		for shortcut, skill in pairs(self.ccSkills) do
			if moebius92.SkillData.shortcut[shortcut] == name then
				skill.activeSkill = askill;
				skill.handlers = { };
				skill.handlers["ResetTimeChanged"] = moebius92.Handlers.AddEventHandler(askill, "ResetTimeChanged", function() self:SkillUse(shortcut); end);
				skill.regimes.cc:SetVisible(true);
				skill.regimes.target:SetVisible(true);
				skill.regimes.options:SetVisible(true);
			end
		end

		for label, config in pairs(self.config) do
			if config.skill ~= nil and moebius92.SkillData.shortcut[config.skill] == name then
				possConfigs[label] = true;
			end
		end
	end

	local t = { };
	for label in pairs(possConfigs) do
		table.insert(t, label);
	end

	if #t == 1 then
		if t[1] ~= self.curConfig then
			self:LoadConfig(t[1]);
			return;
		end
	elseif #t > 1 then
		table.sort(t);
		Turbine.Shell.WriteLine("Multiple possible configs: " .. table.concat(t, ", "));
	end
	self.options:UpdateEntries();
end

function CCTimerWindow:SkillUse(shortcut)
	local time = Turbine.Engine.GetGameTime();
	local cooldown = self.ccSkills[shortcut].activeSkill:GetCooldown();
	local rtime = self.ccSkills[shortcut].activeSkill:GetResetTime();

	if rtime >= 0 and (math.abs(time - (rtime - cooldown)) < 0.1) then
		self.ccSkills[shortcut].useTime = rtime - cooldown;
		if self.ccSkills[shortcut].hitTime ~= nil and (self.ccSkills[shortcut].useTime - self.ccSkills[shortcut].hitTime) < 1.5 then
			if self.time ~= nil then
				self.time.reset = self.ccSkills[shortcut].useTime;
				self:Calibrate();
			end
			self:AddCC(shortcut);
		end
	end
end

function CCTimerWindow:ChatMessage(m)
	m = string.gsub(m, "<[^>]*>", "");
	local i = string.find(m, " ");

	if i ~= nil then
		local name = trim(string.sub(m, 1, i - 1));
		local rest = trim(string.sub(m, i + 1));
		if name == self.name then
			if string.sub(rest, 1, 9) == "scored a " then
				local j = string.find(rest, " with ");
				i = string.find(rest, " on ");
				if i ~= nil then
					local skillName = string.sub(rest, j + 6, i - 1);
					if string.sub(skillName, 1, 9) == "Improved " then
						skillName = string.sub(skillName, 10);
					end
					if string.sub(skillName, 1, 9) == "Advanced " then
						skillName = string.sub(skillName, 10);
					end

					for shortcut, skill in pairs(self.ccSkills) do
						if moebius92.SkillData.shortcut[shortcut] == skillName then
							skill.hitTime = Turbine.Engine.GetGameTime();
							if skill.useTime ~= nil and (skill.hitTime - skill.useTime) < 1.5 then
								if self.time ~= nil then
									self.time.reset = skill.useTime;
									self:Calibrate();
								end
								self:AddCC(shortcut);
							end
						end
					end
				end
			end
		end
	end
end

function CCTimerWindow:AddCC(shortcut)
	local cc = self:NewCCElement(self.regimes.cc);
	cc.type = shortcut;
	cc.startTime = self.ccSkills[shortcut].useTime + self.ccSkills[shortcut].act;
	cc.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, shortcut));
	if self.config[self.curConfig][shortcut].preroot ~= nil then
		cc.endTime = cc.startTime + self.config[self.curConfig][shortcut].preroot;
		cc.pre = true;
	else
		cc.endTime = cc.startTime + self.config[self.curConfig][shortcut].duration;
		cc.pre = false;
	end
	table.insert(self.ccTrack, cc);

	self.ccSkills[shortcut].useTime = nil;
	self.ccSkills[shortcut].hitTime = nil;

	self:AssignLanes();
end

function CCTimerWindow:AssignLanes()
	local time = Turbine.Engine.GetGameTime();

	for _, cc in ipairs(self.ccTrack) do
		if cc.startTime > time then
			cc.top = nil;
			cc.bottom = nil;
		else
			cc.bottom = (cc.endTime - time) * 10;
			cc.top = cc.bottom + self.size;
		end
	end
	moebius92.RaceTrack.layout(self.ccTrack);

	self:UpdateLayout();
end

function CCTimerWindow:UpdateLayout()
	for _, cc in ipairs(self.ccTrack) do
		if cc.lane == nil then
			cc.control:SetVisible(false);
		else
			cc.control:SetVisible(true);
		end
	end

	if self.targetCCName == "Recovering" then
		for _, cc in ipairs(self.ccTrack) do
			if self.ccSkills[cc.type].recovery and math.abs(self.targetCC.endTime - (cc.endTime + 1.0)) < 0.1 then
				self.targetCC.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, cc.type));
				cc.control:SetVisible(false);
				break;
			end
		end
	elseif self.targetCCName ~= nil then
		for _, cc in ipairs(self.ccTrack) do
			if self.ccSkills[cc.type].effect == self.targetCCName and math.abs(self.targetCC.endTime - cc.endTime) < 0.1 then
				self.targetCC.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, cc.type));
				cc.control:SetVisible(false);
				cc.endTime = self.targetCC.endTime
				break;
			end
		end
	end

	for _, effect in ipairs(self.targetTrack) do
		for _, cc in ipairs(self.ccTrack) do
			if effect.type == cc.type and math.abs(effect.endTime - cc.endTime) < 0.1 then
				cc.control:SetVisible(false);
			end
		end
	end

	local w = 0;
	for _, cc in ipairs(self.ccTrack) do
		if cc.control:IsVisible() and cc.lane > w then
			w = cc.lane;
		end
	end

	if w == 0 then
		self:SetWidth(self.size + 4);
		self.regimes.cc:SetVisible(false);
	else
		self:SetWidth(self.size + 4 + 2 + self.size * w + 4);
		self.regimes.cc:SetWidth(self.size * w + 4);
		self.regimes.cc:SetVisible(true);
	end

	if self:IsMouseVisible() or self.targetCCName ~= nil or #self.targetTrack > 0 or (self.regimes.cc:IsVisible() and self.target ~= nil) then
		self.regimes.target:SetVisible(true);
	else
		self.regimes.target:SetVisible(false);
	end

	self:Update();
end

function CCTimerWindow:Update()
	local time = Turbine.Engine.GetGameTime();
	local changed = false;
	local t = { };

	for i, cc in ipairs(self.ccTrack) do
		local delta = cc.endTime - time;

		if delta < -1.0 or (delta < 0.0 and not self.ccSkills[cc.type].recovery) then
			changed = true;
			if cc.pre then
				cc.pre = false;
				cc.endTime = cc.endTime + self.config[self.curConfig][cc.type].duration;
				cc.type = "0x70003EC0";
				cc.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Skill, "0x70003EC0"));
			else
				cc.control:SetParent(nil);
				cc.control:SetVisible(false);
				table.insert(t, i);
			end
		elseif cc.startTime <= time and cc.lane == nil then
			changed = true;
		end
	end

	table.sort(t, function(a, b) return a > b; end);
	for _, i in ipairs(t) do
		table.remove(self.ccTrack, i);
	end

	if changed then
		self:AssignLanes();
		return;
	end

	t = { };
	changed = false;

	for i, effect in ipairs(self.targetTrack) do
		local delta = effect.endTime - time;
		if delta < 0.0 then
			effect.control:SetParent(nil);
			effect.control:SetVisible(false);
			table.insert(t, i);
			changed = true;
		end
	end

	table.sort(t, function(a, b) return a > b; end);
	for _, i in ipairs(t) do
		table.remove(self.targetTrack, i);
	end

	if changed then
		self:UpdateLayout();
		return;
	end

	local update = false;
	for _, cc in ipairs(self.ccTrack) do
		if cc.control:IsVisible() then
			local t = cc.endTime - time;
			cc.control:SetPosition(self.size * (cc.lane - 1) + 2, (self.seconds - t) * 10);
			cc.control:SetBackColor(self.regimes.cc:GetColor((self.seconds - t) * 10 + self.size));
			update = true;
		end
	end

	if self.targetCCName ~= nil then
		local t = self.targetCC.endTime - time;
		if t <= 0.0 then
			self.targetCCName = nil;
			self.targetCCID = nil;
			self.targetCC.control:SetVisible(false);
			self:UpdateLayout();
		elseif self.targetCCName ~= "Recovering" then
			self.targetCC.control:SetPosition(2, (self.seconds - t) * 10);
			self.targetCC.control:SetBackColor(self.regimes.cc:GetColor((self.seconds - t) * 10 + self.size));
		else
			self.targetCC.control:SetPosition(2, (self.seconds + 1.0 - t) * 10);
			self.targetCC.control:SetBackColor(self.regimes.cc:GetColor((self.seconds + 1.0 - t) * 10 + self.size));
		end
		update = true;
	end

	for _, effect in ipairs(self.targetTrack) do
		local t = effect.endTime - time;
		effect.control:SetPosition(2, (self.seconds - t) * 10);
		effect.control:SetBackColor(self.regimes.cc:GetColor((self.seconds - t) * 10 + self.size));
		update = true;
	end

	self:SetWantsUpdates(update);
end

function CCTimerWindow:LoadConfig(config)
	if self.config[config] == nil then
		Turbine.Shell.WriteLine("Config " .. self.curConfig .. " not found.");
	else
		self.curConfig = config;
		self:UpdateRegimes();
		self:SaveSettings();
		self.options:UpdateEntries();
		Turbine.Shell.WriteLine("Config " .. self.curConfig .. " loaded.");
	end
end

function CCTimerWindow:GetConfigs()
	local t = { };

	for label in pairs(self.config) do
		table.insert(t, label);
	end

	table.sort(t);
	return t;
end

function CCTimerWindow:UpdateRegimes()
	for _, regime in pairs(self.regimes) do
		regime:SetColor(self.config[self.curConfig].back.r, self.config[self.curConfig].back.g, self.config[self.curConfig].back.b);
	end
	for _, regime in pairs(self.regimesRecovery) do
		regime:SetColor(self.config[self.curConfig].rec.r, self.config[self.curConfig].rec.g, self.config[self.curConfig].rec.b);
	end

	local config = self.config[self.curConfig];
	for shortcut, skill in pairs(self.ccSkills) do
		for _, regime in pairs(skill.regimes) do
			regime:SetColor(config[shortcut].r, config[shortcut].g, config[shortcut].b);
		end
		local top = config[shortcut].start;
		if top ~= nil then
			top = self.size + (self.seconds - top) * 10;
		end
		local bottom = config[shortcut].stop;
		if bottom ~= nil then
			bottom = self.size + (self.seconds - bottom) * 10;
		end
		for _, regime in pairs(skill.regimes) do
			regime:SetBounds(top, bottom);
		end
	end
end

function CCTimerWindow:ToggleCalibrate()
	if self.time == nil then
		self:StartCalibrate();
	else
		self:StopCalibrate();
	end
end

function CCTimerWindow:StartCalibrate()
	self.time = { };
	self.options:StartCalibrate();
end

function CCTimerWindow:Calibrate()
	if self.time.cc ~= nil and self.time.reset ~= nil then
		local act = self.time.cc - self.time.reset;
		local key;

		Turbine.Shell.WriteLine(string.format("Act: %.3f", act));
		if self.time.key ~= nil then
			key = self.time.cc - self.time.key;
			Turbine.Shell.WriteLine(string.format("Key: %.3f", key));
		end
		self.options:Calibrate(act, key);
		self:StopCalibrate();
	end
end

function CCTimerWindow:StopCalibrate()
	self.time = nil;
	self.options:StopCalibrate();
end

function CCTimerWindow:MouseDown(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		self.dragStartX = args.X;
		self.dragStartY = args.Y;
		self.dragging = true;
	end
end

function CCTimerWindow:MouseMove(args)
	if self.dragging then
		local x, y = self:GetPosition();
		x = x + (args.X - self.dragStartX);
		y = y + (args.Y - self.dragStartY);
		self.x = x;
		self.y = y;
		self:SetPosition(x, y);
	end
end

function CCTimerWindow:MouseUp(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		self:SaveSettings();
		self.dragging = false;
	end
end

function CCTimerWindow:KeyDown(args)
	if args.Action == 0x1000007B then
		if self.dragging then
			self:SaveSettings();
			self.dragging = false;
		end
		self:SetMouseVisible(not self:IsMouseVisible());
		self:UpdateLayout();
	elseif args.Action == 0x100000B3 then
		self:SetVisible(not self:IsVisible());
	elseif keymap[args.Action] ~= nil and self.time ~= nil then
		self.time.key = Turbine.Engine.GetGameTime();
	end
end

function CCTimerWindow:SaveSettings()
	local t = { };

	t.version = self.version;

	t.x = self.x;
	t.y = self.y;
	t.config = self.config;
	t.curConfig = self.curConfig;

	Turbine.PluginData.Save(Turbine.DataScope.Character, "CCTimer", serialize(t));
end

function CCTimerWindow:LoadSettings()
	local t = Turbine.PluginData.Load(Turbine.DataScope.Character, "CCTimer");
	if type(t) == "string" then
		t = loadstring("return " .. t .. ";")();
	end

	if type(t) == "table" then
		local v = snarfversion(t.version);

		if v[1] == 0 and v[2] == 6 then
			self.x = t.x;
			self.y = t.y;
			self.config = t.config;
			self.curConfig = t.curConfig;
		end
	end

	if self.config[self.curConfig] == nil then
		self.curConfig = "default";
		self.config[self.curConfig] = { };
		for shortcut, skill in pairs(self.ccSkills) do
			self.config[self.curConfig][shortcut] = { };
			self.config[self.curConfig][shortcut].duration = skill.duration;
			self.config[self.curConfig][shortcut].act = skill.act;
			self.config[self.curConfig][shortcut].start = skill.start;
			self.config[self.curConfig][shortcut].stop = skill.stop;
			self.config[self.curConfig][shortcut].r = skill.r;
			self.config[self.curConfig][shortcut].g = skill.g;
			self.config[self.curConfig][shortcut].b = skill.b;

			if ccEffects[skill.effect] == nil then
				self.config[self.curConfig][shortcut].preroot = 3;
			end
		end

		self.config[self.curConfig].back = { };
		self.config[self.curConfig].back.r = 0.0;
		self.config[self.curConfig].back.g = 1.0;
		self.config[self.curConfig].back.b = 0.0;

		self.config[self.curConfig].rec = { };
		self.config[self.curConfig].rec.r = 1.0;
		self.config[self.curConfig].rec.g = 0.0;
		self.config[self.curConfig].rec.b = 0.0;
	end
end